unit Transparency;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

const
  LWA_COLORKEY: DWORD = $1;
  LWA_ALPHA: DWORD = $2;
  WS_EX_LAYERED: DWORD = $80000;

type
  TFuncSetLayeredWindowAttributes
    = function(hwnd: HWND; crKey: COLORREF; bAlpha: BYTE; dwFlags: DWORD): BOOL; stdcall;

  TTransparency = class(TComponent)
  private
    OS_W2000 : Boolean;
    FTransparency : Integer;
    FuncSetLayeredWindowAttributes: TFuncSetLayeredWindowAttributes;
    procedure GetTransparencyAvailable;
    procedure SetTransparency(Handle : HWND; Pos : Integer; EffectOn : boolean);
    procedure SetFTransparency(val : Integer);
  protected
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Opacity: Integer read FTransparency write SetFTransparency;
    procedure SetFormTransparency;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('YU Spin', [TTransparency]);
end;

constructor TTransparency.Create(AOwner: TComponent); { this goes after implementation }
begin
  inherited Create(AOwner);
  FTransparency := 100;
  GetTransparencyAvailable;
  SetTransparency((Self.Owner as TForm).Handle, FTransparency, True);
end;

procedure TTransparency.GetTransparencyAvailable;
var
  VersionInfo: TOSVERSIONINFO;
begin
  VersionInfo.dwOSVersionInfoSize := sizeof(TOSVERSIONINFO);
  GetVersionEx(VersionInfo);

  if not ((VersionInfo.dwPlatformId = VER_PLATFORM_WIN32_NT) and (VersionInfo.dwMajorVersion >= 5))
    then OS_W2000 := False
    else OS_W2000 := True;

  if (OS_W2000) then  //----- Use Windows 2000 SetLayeredWindowAttributes
  begin
    FuncSetLayeredWindowAttributes := GetProcAddress(GetModuleHandle('user32.dll'), 'SetLayeredWindowAttributes');
  end else
  begin               //----- Use Windows 95/98 dummy func
  end;
end;

procedure TTransparency.SetTransparency(Handle : HWND; Pos : Integer; EffectOn : boolean);
begin
  if (OS_W2000) then
  if (EffectOn) and (Pos < 255) THEN
    begin //----- ON
      SetWindowLong(Handle, GWL_EXSTYLE, Dword(GetWindowLong (Handle, GWL_EXSTYLE)) or WS_EX_LAYERED);
      FuncSetLayeredWindowAttributes(Handle, clSilver, Pos, LWA_ALPHA);
    end
  else
    begin //----- OFF
      SetWindowLong(Handle, GWL_EXSTYLE, DWORD(GetWindowLong(Handle, GWL_EXSTYLE)) and not WS_EX_LAYERED)
    end;
end;

procedure TTransparency.SetFormTransparency;
begin
  GetTransparencyAvailable;
  SetTransparency((Self.Owner as TForm).Handle, FTransparency, True);
end;

procedure TTransparency.SetFTransparency(val: Integer);
begin
  FTransparency := val;
  GetTransparencyAvailable;
  SetTransparency((Self.Owner as TForm).Handle, FTransparency, True);
end;

end.
